@@ -1,21 +1,20 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
+   This file was written by Peter Miller <millerp@canb.auug.org.au>
 
-   This file was written by Peter Miller <pmiller@agso.gov.au>
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -140,7 +139,14 @@ main (argc, argv)
   /* Version information is requested.  */
   if (do_version)
     {
-      fprintf (stderr, "%s - GNU %s %s\n", program_name, PACKAGE, VERSION);
+      printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
+      /* xgettext: no-wrap */
+      printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+"),
+	      "1995, 1996, 1997, 1998");
+      printf (_("Written by %s.\n"), "Peter Miller");
       exit (EXIT_SUCCESS);
     }
 
@@ -172,11 +178,12 @@ usage (status)
      int status;
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information\n"),
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
-    /* xgettext: no-wrap */
-    printf (_("\
+    {
+      /* xgettext: no-wrap */
+      printf (_("\
 Usage: %s [OPTION] def.po ref.po\n\
 Mandatory arguments to long options are mandatory for short options too.\n\
   -D, --directory=DIRECTORY   add DIRECTORY to list for input files search\n\
@@ -189,7 +196,9 @@ old translations.  The ref.po file is the last created PO file\n\
 (generally by xgettext).  This is useful for checking that you have\n\
 translated each and every message in your program.  Where an exact match\n\
 cannot be found, fuzzy matching is used to produce better diagnostics.\n"),
-	    program_name);
+	      program_name);
+      fputs (_("Report bugs to <bug-gnu-utils@gnu.org>.\n"), stdout);
+    }
 
   exit (status);
 }
@@ -345,7 +354,7 @@ compare_directive_message (that, msgid, msgid_pos, msgstr, msgstr_pos)
     {
       gram_error_at_line (msgid_pos, _("duplicate message definition"));
       gram_error_at_line (&mvp->pos, _("\
-this is the location of the first definition"));
+...this is the location of the first definition"));
       free (msgstr);
     }
   else
