@@ -1,20 +1,20 @@
-/* msgfmt - converts Uniforum style .po files to binary .mo files
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+/* Converts Uniforum style .po files to binary .mo files
+   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -167,6 +167,7 @@ static void format_directive_message PARAMS ((po_ty *__pop, char *__msgid,
 					      char *__msgstr,
 					      lex_pos_ty *__msgstr_pos));
 static void format_comment_special PARAMS ((po_ty *pop, const char *s));
+static void format_debrief PARAMS((po_ty *));
 static struct msg_domain *new_domain PARAMS ((const char *name));
 static int compare_id PARAMS ((const void *pval1, const void *pval2));
 static void write_table PARAMS ((FILE *output_file, hash_table *tab));
@@ -187,7 +188,7 @@ main(argc, argv)
   int strict_uniforum = 0;
 
   /* Set default value for global variables.  */
-  alignment = DEFAULT_ALIGNMENT;
+  alignment = DEFAULT_OUTPUT_ALIGNMENT;
 
   /* Set program name for messages.  */
   program_name = argv[0];
@@ -251,7 +252,14 @@ main(argc, argv)
   /* Version information is requested.  */
   if (do_version)
     {
-      fprintf (stderr, "%s - GNU %s %s\n", program_name, PACKAGE, VERSION);
+      printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
+      /* xgettext: no-wrap */
+      printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+"),
+	      "1995, 1996, 1997, 1998");
+      printf (_("Written by %s.\n"), "Ulrich Drepper");
       exit (EXIT_SUCCESS);
     }
 
@@ -350,12 +358,15 @@ usage (status)
      int status;
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information\n"),
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
-    /* xgettext: no-wrap */
-    printf (_("\
+    {
+      /* xgettext: no-wrap */
+      printf (_("\
 Usage: %s [OPTION] filename.po ...\n\
+Generate binary message catalog from textual translation description.\n\
+\n\
 Mandatory arguments to long options are mandatory for short options too.\n\
   -a, --alignment=NUMBER      align strings to NUMBER bytes (default: %d)\n\
   -c, --check                 perform language dependent checks on strings\n\
@@ -373,7 +384,9 @@ Giving the -v option more than once increases the verbosity level.\n\
 \n\
 If input file is -, standard input is read.  If output file is -,\n\
 output is written to standard output.\n"),
-	    program_name, DEFAULT_ALIGNMENT);
+	      program_name, DEFAULT_OUTPUT_ALIGNMENT);
+      fputs (_("Report bugs to <bug-gnu-utils@gnu.org>.\n"), stdout);
+    }
 
   exit (status);
 }
@@ -435,7 +448,7 @@ format_debrief (that)
 
   /* If in verbose mode, test whether header entry was found.  */
   if (verbose_level > 0 && this->has_header_entry == 0)
-    error (0, 0, _("warning: no header entry found"));
+    error (0, 0, _("%s: warning: no header entry found"), gram_pos.file_name);
 }
 
 
@@ -509,8 +522,11 @@ format_directive_message (that, msgid_string, msgid_pos, msgstr_string,
       /* Free strings allocated in po-gram.y.  */
       free (msgstr_string);
 
-      /* Increment counter for untranslated messages.  */
-      ++msgs_untranslated;
+      /* Increment counter for fuzzy/untranslated messages.  */
+      if (this->is_fuzzy)
+	++msgs_fuzzy;
+      else
+	++msgs_untranslated;
 
       goto prepare_next;
     }
@@ -554,15 +570,17 @@ header field `%s' should start at beginning of line"),
 		       && strncmp (default_values[cnt],
 				   endp + strlen (required_fields[cnt]) + 2,
 				   strlen (default_values[cnt])) == 0)
-		if (initial != -1)
-		  {
-		    error (0, 0, _("\
+		{
+		  if (initial != -1)
+		    {
+		      error (0, 0, _("\
 some header fields still have the initial default value"));
-		    initial = -1;
-		    break;
-		  }
-		else
-		  initial = cnt;
+		      initial = -1;
+		      break;
+		    }
+		  else
+		    initial = cnt;
+		}
 	    }
 
 	  if (initial != -1)
@@ -601,23 +619,28 @@ some header fields still have the initial default value"));
     {
       /* We don't need the just constructed entry.  */
       free (entry);
-      /* And the parameter string (allocated in po-gram.y).  */
-      free (msgstr_string);
 
       if (verbose_level > 0)
 	{
-	  /* We give a warning about this.  Get the old definition for
-	     reference.  */
+	  /* We give a fatal error about this, but only if the
+	     translations are different.  Tell the user the old
+	     definition for reference.  */
 	  find_entry (&current_domain->symbol_tab, msgid_string,
 		      strlen (msgid_string), (void **) &entry);
+	  if (0 != strcmp(msgstr_string, entry->msgstr))
+	    {
+	      gram_error_at_line (msgid_pos, _("duplicate message definition"));
+	      gram_error_at_line (&entry->pos, _("\
+...this is the location of the first definition"));
 
-	  gram_error_at_line (msgid_pos, _("duplicate message ID"));
-	  gram_error_at_line (&entry->pos, _("\
-this is the location of the first definition"));
-
-	  /* FIXME Should this be always a reason for an exit status != 0?  */
-	  exit_status = EXIT_FAILURE;
+	      /* FIXME Should this be always a reason for an exit status != 0?  */
+	      exit_status = EXIT_FAILURE;
+	    }
 	}
+
+      /* We don't need the just constructed entries'
+         parameter string (allocated in po-gram.y).  */
+      free (msgstr_string);
     }
 
 prepare_next:
@@ -647,7 +670,8 @@ format_comment_special (that, s)
 	{
 	  warned = 1;
 	  error (0, 0, _("\
-WARNING: source file contains fuzzy translation"));
+%s: warning: source file contains fuzzy translation"),
+                 gram_pos.file_name);
 	}
 
       this->is_fuzzy = 1;
@@ -848,7 +872,7 @@ check_pair (msgid, msgid_pos, msgstr, msgstr_pos, is_format)
   if (((msgid[0] == '\n') ^ (msgstr[0] == '\n')) != 0)
     {
       error_at_line (0, 0, msgid_pos->file_name, msgid_pos->line_number, _("\
-`msgid' and `msgstr' entries does not both begin with '\\n'"));
+`msgid' and `msgstr' entries do not both begin with '\\n'"));
       exit_status = EXIT_FAILURE;
     }
 
@@ -856,7 +880,7 @@ check_pair (msgid, msgid_pos, msgstr, msgstr_pos, is_format)
   if (((msgid[msgid_len - 1] == '\n') ^ (msgstr[msgstr_len - 1] == '\n')) != 0)
     {
       error_at_line (0, 0, msgid_pos->file_name, msgid_pos->line_number, _("\
-`msgid' and `msgstr' entries does not both end with '\\n'"));
+`msgid' and `msgstr' entries do not both end with '\\n'"));
       exit_status = EXIT_FAILURE;
     }
 
@@ -895,7 +919,7 @@ format specifications for argument %u are not the same"), cnt);
 }
 
 
-/* So that the one parer can be used for multiple programs, and also
+/* So that the one parser can be used for multiple programs, and also
    use good data hiding and encapsulation practices, an object
    oriented approach has been taken.  An object instance is allocated,
    and all actions resulting from the parse will be through
