@@ -1,20 +1,20 @@
 /* msgunfmt - converts binary .mo files to Uniforum style .po files
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -58,10 +58,14 @@ extern int errno;
 /* String containing name the program is called with.  */
 const char *program_name;
 
+/* Force output of PO file even if empty.  */
+static int force_po;
+
 /* Long options.  */
 static const struct option long_options[] =
 {
   { "escape", no_argument, NULL, 'E' },
+  { "force-po", no_argument, &force_po, 1 },
   { "help", no_argument, NULL, 'h' },
   { "indent", no_argument, NULL, 'i' },
   { "no-escape", no_argument, NULL, 'e' },
@@ -166,7 +170,14 @@ main (argc, argv)
   /* Version information is requested.  */
   if (do_version)
     {
-      fprintf (stderr, "%s - GNU %s %s\n", program_name, PACKAGE, VERSION);
+      printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
+      /* xgettext: no-wrap */
+      printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+"),
+	      "1995, 1996, 1997, 1998");
+      printf (_("Written by %s.\n"), "Ulrich Drepper");
       exit (EXIT_SUCCESS);
     }
 
@@ -183,7 +194,7 @@ main (argc, argv)
     mlp = read_mo_file (NULL, "-");
 
   /* Write the resulting message list to the given .po file.  */
-  message_list_print (mlp, output_file, 0, 0);
+  message_list_print (mlp, output_file, force_po, 0);
 
   /* No problems.  */
   exit (EXIT_SUCCESS);
@@ -196,7 +207,7 @@ usage (status)
      int status;
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information\n"),
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
@@ -206,6 +217,7 @@ Usage: %s [OPTION] [FILE]...\n\
 Mandatory arguments to long options are mandatory for short options too.\n\
   -e, --no-escape          do not use C escapes in output (default)\n\
   -E, --escape             use C escapes in output, no extended chars\n\
+      --force-po           write PO file even if empty\n\
   -h, --help               display this help and exit\n\
   -i, --indent             write indented output style\n\
   -o, --output-file=FILE   write output into FILE instead of standard output\n\
@@ -218,7 +230,8 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 Convert binary .mo files to Uniforum style .po files.\n\
 Both little-endian and big-endian .mo files are handled.\n\
 If no input file is given or it is -, standard input is read.\n\
-By default the output is written to standard output.\n"),
+By default the output is written to standard output.\n"), stdout);
+      fputs (_("Report bugs to <bug-gnu-utils@gnu.org>.\n"),
 	     stdout);
     }
 
