@@ -1,21 +1,21 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
 
-   This file was written by Peter Miller <pmiller@agso.gov.au>
+   This file was written by Peter Miller <millerp@canb.auug.org.au>
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -58,12 +58,12 @@ extern int errno;
    5. Parse each resulting logical line as preprocessing tokens a
       white space.
 
-   6. Recognise and carry out directives (it also expands macros on
+   6. Recognize and carry out directives (it also expands macros on
       non-directive lines, which we do not do here).
 
    7. Replaces escape sequences within character strings with their
-      single character equivanents (we do this in step 5, because we
-      dont have to worry about the #include argument).
+      single character equivalents (we do this in step 5, because we
+      don't have to worry about the #include argument).
 
    8. Concatenates adjacent string literals to form single string
       literals (because we don't expand macros, there are a few things
@@ -119,7 +119,7 @@ static int last_non_comment_line = -1;
 static int newline_count = 0;
 
 
-/* Prototypes for lcoal functions.  */
+/* Prototypes for local functions.  */
 static int phase1_getc PARAMS ((void));
 static void phase1_ungetc PARAMS ((int __c));
 static int phase2_getc PARAMS ((void));
@@ -222,7 +222,7 @@ xgettext_lex_close ()
 
 /* 1. Terminate line by \n, regardless of the external representation of
    a text line.  Stdio does this for us, we just need to check that
-   there are no I/O errors, and cope with potentially 2 chacarters of
+   there are no I/O errors, and cope with potentially 2 characters of
    pushback, not just the one that ungetc can cope with.  */
 
 /* Maximum used guaranteed to be < 4.  */
@@ -533,7 +533,7 @@ phase4_ungetc (c)
 
 
 /* 7. Replace escape sequences within character strings with their
-   single character equivanents.  This is called from phase 5, because
+   single character equivalents.  This is called from phase 5, because
    we don't have to worry about the #include argument.  There are
    pathological cases which could bite us (like the DOS directory
    separator), but just pretend it can't happen.  */
@@ -549,8 +549,27 @@ phase7_getc ()
 
   /* Use phase 3, because phase 4 elides comments.  */
   c = phase3_getc ();
+
+  /* Return a magic newline indicator, so that we can distinguish
+     between the user requesting a newline in the string (e.g. using
+     "\n" or "\15") from the user failing to terminate the string or
+     character constant.  The ANSI C standard says: 3.1.3.4 Character
+     Constants contain ``any character except single quote, backslash or
+     newline; or an escape sequence'' and 3.1.4 String Literals contain
+     ``any character except double quote, backslash or newline; or an
+     escape sequence''.
+
+     Most compilers give a fatal error in this case, however gcc is
+     stupidly silent, even though this is a very common typo.  OK, so
+     gcc --pedantic will tell me, but that gripes about too much other
+     stuff.  Could I have a ``gcc -Wnewline-in-string'' option, or
+     better yet a ``gcc -fno-newline-in-string'' option, please?  Gcc is
+     also inconsistent between string literals and character constants:
+     you may not embed newlines in character constants; try it, you get
+     a useful diagnostic.  --PMiller  */
   if (c == '\n')
     return P7_NEWLINE;
+
   if (c == '"')
     return P7_QUOTES;
   if (c == '\'')
@@ -575,10 +594,18 @@ phase7_getc ()
       /* The \a and \v escapes were added by the ANSI C Standard.
 	 Prior to the Standard, most compilers did not have them.
 	 Because we need the same program on all platforms we don't
-	 provide support for them here.  */
+	 provide support for them here.
+
+	 The gcc sources comment that \a is commonly available in
+	 pre-ANSI compilers.  --PMiller  */
 
     case 'b':
       return '\b';
+
+      /* The \e escape is preculiar to gcc, and assumes an ASCII
+         character set (or superset).  We don't provide support for it
+         here.  */
+
     case 'f':
       return '\f';
     case 'n':
@@ -790,7 +817,7 @@ phase5_get (tp)
 
     case '0': case '1': case '2': case '3': case '4':
     case '5': case '6': case '7': case '8': case '9':
-      /* The preprecessing number token is more "generous" than the C
+      /* The preprocessing number token is more "generous" than the C
 	 number tokens.  This is mostly due to token pasting (another
 	 thing we can ignore here).  */
       bufpos = 0;
@@ -862,6 +889,8 @@ phase5_get (tp)
 	  c = phase7_getc ();
 	  if (c == P7_NEWLINE)
 	    {
+	      error (0, 0, _("%s:%d: warning: unterminated character constant"),
+	        logical_file_name, line_number - 1);
 	      phase7_ungetc ('\n');
 	      break;
 	    }
@@ -882,6 +911,8 @@ phase5_get (tp)
 	  c = phase7_getc ();
 	  if (c == P7_NEWLINE)
 	    {
+	      error (0, 0, _("%s:%d: warning: unterminated string literal"),
+	        logical_file_name, line_number - 1);
 	      phase7_ungetc ('\n');
 	      break;
 	    }
@@ -921,7 +952,7 @@ phase5_get (tp)
     default:
       /* We could carefully recognize each of the 2 and 3 character
         operators, but it is not necessary, as we only need to recognize
-        gettext invokations.  Don't bother.  */
+        gettext invocations.  Don't bother.  */
       tp->type = token_type_symbol;
       return;
     }
@@ -986,7 +1017,7 @@ phaseX_get (tp)
 	    }
 
 	  /* Discard the leading white space token, the hash is all
-	     phase 6 is intersted in.  */
+	     phase 6 is interested in.  */
 	  if (tp->type != token_type_eof && tp->type != token_type_eoln)
 	    middle = 1;
 	  break;
@@ -999,7 +1030,7 @@ phaseX_get (tp)
 }
 
 
-/* 6. Recognise and carry out directives (it also expands macros on
+/* 6. Recognize and carry out directives (it also expands macros on
    non-directive lines, which we do not do here).  The only directive
    we care about is the #line directive.  We throw all the others
    away.  */
