@@ -1,20 +1,20 @@
-/* xgettext - extracts strings from C source file to Uniforum style .po file
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+/* Extracts strings from C source file to Uniforum style .po file.
+   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <ctype.h>
 #include <errno.h>
 #include <getopt.h>
+#include <sys/param.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <time.h>
@@ -74,13 +75,13 @@ static int add_all_comments;
 static int line_comment;
 
 /* Tag used in comment of prevailing domain.  */
-static char *comment_tag;
+static unsigned char *comment_tag;
 
 /* Name of default domain file.  If not set defaults to messages.po.  */
 static char *default_domain;
 
 /* If called with --debug option the output reflects whether format
-   string recognision is done automatically or forced by the user.  */
+   string recognition is done automatically or forced by the user.  */
 static int do_debug;
 
 /* Content of .po files with symbols to be excluded.  */
@@ -92,6 +93,10 @@ static int extract_all;
 /* Force output of PO file even if empty.  */
 static int force_po;
 
+/* If nonzero a non GNU related user wants to use this.  Omit the FSF
+   copyright in the output.  */
+static int foreign_user;
+
 /* String used as prefix for msgstr.  */
 static char *msgstr_prefix;
 
@@ -125,15 +130,18 @@ static const struct option long_options[] =
   { "extract-all", no_argument, &extract_all, 1 },
   { "files-from", required_argument, NULL, 'f' },
   { "force-po", no_argument, &force_po, 1 },
+  { "foreign-user", no_argument, &foreign_user, 1 },
   { "help", no_argument, NULL, 'h' },
   { "indent", no_argument, NULL, 'i' },
   { "join-existing", no_argument, NULL, 'j' },
   { "keyword", optional_argument, NULL, 'k' },
+  { "language", required_argument, NULL, 'L' },
   { "msgstr-prefix", optional_argument, NULL, 'm' },
   { "msgstr-suffix", optional_argument, NULL, 'M' },
   { "no-escape", no_argument, NULL, 'e' },
   { "no-location", no_argument, &line_comment, 0 },
   { "omit-header", no_argument, &omit_header, 1 },
+  { "output", required_argument, NULL, 'o' },
   { "output-dir", required_argument, NULL, 'p' },
   { "sort-by-file", no_argument, NULL, 'F' },
   { "sort-output", no_argument, NULL, 's' },
@@ -183,6 +191,16 @@ static message_ty *construct_header PARAMS ((void));
 static enum is_c_format test_whether_c_format PARAMS ((const char *__s));
 
 
+/* The scanners must all be functions returning void and taking one
+   string argument and a message list argument.  */
+typedef void (*scanner_fp) PARAMS ((const char *, message_list_ty *));
+
+static void scanner_c PARAMS ((const char *, message_list_ty *));
+static void scanner_cxx PARAMS ((const char *, message_list_ty *));
+static const char *extension_to_language PARAMS ((const char *));
+static scanner_fp language_to_scanner PARAMS ((const char *));
+
+
 int
 main (argc, argv)
      int argc;
@@ -199,7 +217,8 @@ main (argc, argv)
   char *file_name;
   const char *files_from = NULL;
   string_list_ty *file_list;
-  int input_files_are_cplusplus = 0;
+  char *output_file = NULL;
+  scanner_fp scanner = NULL;
 
   /* Set program name for messages.  */
   program_name = argv[0];
@@ -220,7 +239,7 @@ main (argc, argv)
   default_domain = MESSAGE_DOMAIN_DEFAULT;
 
   while ((optchar = getopt_long (argc, argv,
-				 "ac::Cd:D:eEf:Fhijk::l:m::M::np:sTVw:x:",
+				 "ac::Cd:D:eEf:Fhijk::l:L:m::M::no:p:sTVw:x:",
 				 long_options, NULL)) != EOF)
     switch (optchar)
       {
@@ -245,7 +264,7 @@ main (argc, argv)
 	  }
 	break;
       case 'C':
-	input_files_are_cplusplus = 1;
+	scanner = language_to_scanner ("C++");
 	break;
       case 'd':
 	default_domain = optarg;
@@ -286,6 +305,9 @@ main (argc, argv)
 	    warn_id_len = tmp_val;
 	}
 	break;
+      case 'L':
+	scanner = language_to_scanner (optarg);
+	break;
       case 'm':
 	/* -m takes an optional argument.  If none is given "" is assumed. */
 	msgstr_prefix = optarg == NULL ? "" : optarg;
@@ -297,6 +319,9 @@ main (argc, argv)
       case 'n':
 	line_comment = 1;
 	break;
+      case 'o':
+	output_file = optarg;
+	break;
       case 'p':
 	{
 	  size_t len = strlen (optarg);
@@ -364,7 +389,14 @@ main (argc, argv)
   /* Version information requested.  */
   if (do_version)
     {
-      fprintf (stderr, "%s - GNU %s %s\n", program_name, PACKAGE, VERSION);
+      printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
+      /* xgettext: no-wrap */
+      printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+"),
+	      "1995, 1996, 1997, 1998");
+      printf (_("Written by %s.\n"), "Ulrich Drepper");
       exit (EXIT_SUCCESS);
     }
 
@@ -387,18 +419,32 @@ main (argc, argv)
 
   /* Default output directory is the current directory.  */
   if (output_dir == NULL)
-    output_dir = "./";
+    output_dir = ".";
 
   /* Construct the name of the ouput file.  If the default domain has
      the special name "-" we write to stdout.  */
-  if (strcmp (default_domain, "-") == 0)
+  if (output_file)
+    {
+      if (output_file[0] == '/' ||
+	  strcmp(output_dir, ".") == 0 || strcmp(output_file, "-") == 0)
+	file_name = xstrdup (output_file);
+      else
+	{
+	  /* Please do NOT add a .po suffix! */
+	  file_name = xmalloc (strlen (output_dir)
+			       + strlen (default_domain) + 2);
+	  stpcpy (stpcpy (stpcpy (file_name, output_dir), "/"), output_file);
+	}
+    }
+  else if (strcmp (default_domain, "-") == 0)
     file_name = "-";
   else
     {
       file_name = (char *) xmalloc (strlen (output_dir)
 				    + strlen (default_domain)
-				    + sizeof (".po"));
-      stpcpy (stpcpy (stpcpy (file_name, output_dir), default_domain), ".po");
+				    + sizeof (".po") + 2);
+      stpcpy (stpcpy (stpcpy (stpcpy (file_name, output_dir), "/"),
+		      default_domain), ".po");
     }
 
   /* Determine list of files we have to process.  */
@@ -425,40 +471,42 @@ main (argc, argv)
   /* Process all input files.  */
   for (cnt = 0; cnt < file_list->nitems; ++cnt)
     {
-      /* Determine input file type, Locate, and open it.  */
-      const char *fname = file_list->item[cnt];
-      size_t fname_len = strlen (fname);
+      const char *fname;
+      scanner_fp scan_file;
 
-#define HAS_EXTENSION(ext)						      \
-      (fname_len >= sizeof (ext)					      \
-       && strcmp (&fname[fname_len - sizeof (ext) + 1], ext) == 0)
+      fname = file_list->item[cnt];
 
-      /* Test for C/C++ file.  */
-      if (HAS_EXTENSION (".cc") || HAS_EXTENSION (".C")
-	  || HAS_EXTENSION (".cxx"))
-	scan_c_file (fname, mlp, 1);
-      /* Test for PO files.  */
-      else if (HAS_EXTENSION (".po") || HAS_EXTENSION (".pot")
-	       || HAS_EXTENSION (".pox"))
-	read_po_file (fname, mlp);
-      /* Here will follow more file types: .sh, .pl, .tcl ... */
+      if (scanner)
+        scan_file = scanner;
       else
 	{
-	  if (!HAS_EXTENSION (".c") && !HAS_EXTENSION (".h"))
-	    {
-	      const char *extp = &fname[fname_len];
+	  const char *extension;
+	  const char *language;
 
-	      while (extp[-1] != '.' && (extp[0] == '\0' || extp[-1] != '/'))
-		--extp;
+	  /* Work out what the file extension is.  */
+	  extension = strrchr (fname, '/');
+	  if (!extension)
+	    extension = fname;
+	  extension = strrchr (extension, '.');
+	  if (extension)
+	    ++extension;
+	  else
+	    extension = "";
 
-	      error (0, 0, _("\
-warning: type of file `%s' with extension `%s' is unknown; will try C"),
-		     fname, extp);
-	    }
-
-	  /* The default case: C files.  */
-	  scan_c_file (fname, mlp, input_files_are_cplusplus);
+	  /* derive the language from the extension, and the scanner
+	     function from the language.  */
+	  language = extension_to_language (extension);
+	  if (language == NULL)
+	  {
+	    error (0, 0, _("\
+warning: file `%s' extension `%s' is unknown; will try C"), fname, extension);
+	    language = "C";
+	  }
+	  scan_file = language_to_scanner (language);
 	}
+
+      /* Scan the file.  */
+      scan_file (fname, mlp);
     }
   string_list_free (file_list);
 
@@ -481,18 +529,20 @@ usage (status)
      int status;
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information\n"),
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
       /* xgettext: no-wrap */
       printf (_("\
 Usage: %s [OPTION] INPUTFILE ...\n\
+Extract translatable string from given input files.\n\
+\n\
 Mandatory arguments to long options are mandatory for short options too.\n\
   -a, --extract-all              extract all strings\n\
   -c, --add-comments[=TAG]       place comment block with TAG (or those\n\
                                  preceding keyword lines) in output file\n\
-  -C, --c++                      recognize C++ style comments\n\
+  -C, --c++                      shorthand for --language=C++\n\
       --debug                    more detailed formatstring recognision result\n\
   -d, --default-domain=NAME      use NAME.po for output (instead of messages.po)\n\
   -D, --directory=DIRECTORY      add DIRECTORY to list for input files search\n\
@@ -500,6 +550,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -E, --escape                   use C escapes in output, no extended chars\n\
   -f, --files-from=FILE          get list of input files from FILE\n\
       --force-po                 write PO file even if empty\n\
+      --foreign-user             omit FSF copyright in output for foreign user\n\
   -F, --sort-by-file             sort output by file location\n"),
 	      program_name);
       /* xgettext: no-wrap */
@@ -510,6 +561,8 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -k, --keyword[=WORD]           additonal keyword to be looked for (without\n\
                                  WORD means not to use default keywords)\n\
   -l, --string-limit=NUMBER      set string length limit to NUMBER instead %u\n\
+  -L, --language=NAME            recognise the specified language (C, C++, PO),\n\
+                                 otherwise is guessed from file extension\n\
   -m, --msgstr-prefix[=STRING]   use STRING or \"\" as prefix for msgstr entries\n\
   -M, --msgstr-suffix[=STRING]   use STRING or \"\" as suffix for msgstr entries\n\
       --no-location              do not write '#: filename:line' lines\n"),
@@ -518,6 +571,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
       fputs (_("\
   -n, --add-location             generate '#: filename:line' lines (default)\n\
       --omit-header              don't write header with `msgid \"\"' entry\n\
+  -o, --output=FILE              write output to specified file\n\
   -p, --output-dir=DIR           output files will be placed in directory DIR\n\
   -s, --sort-output              generate sorted output and remove duplicates\n\
       --strict                   write out strict Uniforum conforming .po file\n\
@@ -526,7 +580,8 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -w, --width=NUMBER             set output page width\n\
   -x, --exclude-file=FILE        entries from FILE are not extracted\n\
 \n\
-If INPUTFILE is -, standard input is read.\n"),
+If INPUTFILE is -, standard input is read.\n"), stdout);
+      fputs (_("Report bugs to <bug-gnu-utils@gnu.org>.\n"),
 	     stdout);
     }
 
@@ -635,7 +690,7 @@ exclude_directive_message (pop, msgid, msgid_pos, msgstr, msgstr_pos)
 }
 
 
-/* So that the one parer can be used for multiple programs, and also
+/* So that the one parser can be used for multiple programs, and also
    use good data hiding and encapsulation practices, an object
    oriented approach has been taken.  An object instance is allocated,
    and all actions resulting from the parse will be through
@@ -788,7 +843,7 @@ scan_c_file(filename, mlp, is_cpp_file)
   /* The file is broken into tokens.  Scan the token stream, looking for
      a keyword, followed by a left paren, followed by a string.  When we
      see this sequence, we have something to remember.  We assume we are
-     looking at a valid C or C++ program, and leave the complains about
+     looking at a valid C or C++ program, and leave the complaints about
      the grammar to the compiler.  */
   xgettext_lex_open (filename);
 
@@ -929,12 +984,30 @@ extract_directive_message (that, msgid, msgid_pos, msgstr, msgstr_pos)
   message_variant_ty *mvp;
   size_t j;
 
+  /* See whether we shall exclude this message.  */
+  if (exclude != NULL && message_list_search (exclude, msgid) != NULL)
+    goto discard;
+
   /* If the msgid is the empty string, it is the old header.
      Throw it away, we have constructed a new one.  */
   if (*msgid == '\0')
     {
+      discard:
       free (msgid);
       free (msgstr);
+      if (this->comment != NULL)
+	string_list_free (this->comment);
+      if (this->comment_dot != NULL)
+	string_list_free (this->comment_dot);
+      if (this->filepos != NULL)
+	free (this->filepos);
+      this->comment = NULL;
+      this->comment_dot = NULL;
+      this->filepos_count = 0;
+      this->filepos = NULL;
+      this->is_fuzzy = 0;
+      this->is_c_format = undecided;
+      this->do_wrap = undecided;
       return;
     }
 
@@ -1143,7 +1216,12 @@ construct_header ()
 
   mp = message_alloc ("");
 
-  message_comment_append (mp, "\
+  if (foreign_user)
+    message_comment_append (mp, "\
+SOME DESCRIPTIVE TITLE.\n\
+FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.\n");
+  else
+    message_comment_append (mp, "\
 SOME DESCRIPTIVE TITLE.\n\
 Copyright (C) YEAR Free Software Foundation, Inc.\n\
 FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.\n");
@@ -1212,3 +1290,94 @@ test_whether_c_format (s)
 
   return possible;
 }
+
+
+static void
+  scanner_c (filename, mlp)
+  const char *filename;
+  message_list_ty *mlp;
+{
+  scan_c_file (filename, mlp, 0);
+}
+
+
+static void
+scanner_cxx (filename, mlp)
+  const char *filename;
+  message_list_ty *mlp;
+{
+  scan_c_file (filename, mlp, 1);
+}
+
+
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+#define ENDOF(a) ((a) + SIZEOF(a))
+
+
+static scanner_fp
+language_to_scanner (name)
+  const char *name;
+{
+  typedef struct table_ty table_ty;
+  struct table_ty
+  {
+    const char *name;
+    scanner_fp func;
+  };
+
+  static table_ty table[] =
+  {
+    { "C", scanner_c, },
+    { "C++", scanner_cxx, },
+    { "PO", read_po_file, },
+    /* Here will follow more languages and their scanners: awk, perl,
+       etc...  Make sure new scanners honor the --exlude-file option.  */
+  };
+
+  table_ty *tp;
+
+  for (tp = table; tp < ENDOF(table); ++tp)
+  {
+    if (strcasecmp(name, tp->name) == 0)
+      return tp->func;
+  }
+  error (EXIT_FAILURE, 0, _("language `%s' unknown"), name);
+  /* NOTREACHED */
+  return NULL;
+}
+
+
+static const char *
+extension_to_language (extension)
+  const char *extension;
+{
+  typedef struct table_ty table_ty;
+  struct table_ty
+  {
+    const char *extension;
+    const char *language;
+  };
+
+  static table_ty table[] =
+  {
+    { "c",      "C",    },
+    { "C",      "C++",  },
+    { "c++",    "C++",  },
+    { "cc",     "C++",  },
+    { "cxx",    "C++",  },
+    { "h",      "C",    },
+    { "po",     "PO",   },
+    { "pot",    "PO",   },
+    { "pox",    "PO",   },
+    /* Here will follow more file extensions: sh, pl, tcl ... */
+  };
+
+  table_ty *tp;
+
+  for (tp = table; tp < ENDOF(table); ++tp)
+  {
+    if (strcmp(extension, tp->extension) == 0)
+    return tp->language;
+  }
+  return NULL;
+}
